local _G = _G

-- addon name and namespace
local ADDON, NS = ...

local Addon = LibStub("AceAddon-3.0"):GetAddon(ADDON)

-- module handling
NS._modules  = {}

local EMPTY_TABLE = {}

local tablestack = setmetatable({}, {__mode = 'k'})

local function new()
	local t = next(tablestack)
	if t then
		cache[t] = nil
		return t
	end
	
	return {}
end

local function del(t)
	if type(t) ~= "table" then
		return
	end

	NS:ClearTable(t)
	
	tablestack[t] = true
end

function NS:RegisterModule(module)
	if not module or not NS:IsValidModule(module) then
		return
	end
	
	local type = module:GetType()
	
	if not self._modules[type] then
		self._modules[type] = new()
	end
	
	local name = module:GetName()
	
	if self._modules[type][name] then
		return
	end
	
	self._modules[type][name] = module
	
	return module
end

function NS:UnregisterModule(module)
	if not module or not NS:IsValidModule(module) then
		return
	end

	local type = module:GetType()
	
	if not self._modules[type] then
		return
	end

	self._modules[type][module:GetName()] = nil
	
	if not next(self._modules[type]) then
		del(self._modules[type])
		
		self._modules[type] = nil
	end
end

function NS:IsRegisteredModule(module)
	if not module or not NS:IsValidModule(module) then
		return false
	end
		
	return NS:GetModule(module:GetType(), module:GetName()) and true or false
end

function NS:GetModule(type, name)
	if not self._modules[type] then
		return nil
	end
	
	return self._modules[type][name]
end

function NS:IsValidModule(module)
	if not module or type(module) ~= "table" then
		return false
	end

	if not module.GetType or
		not module.GetName or
		not module.GetDescription or
		not module.IsReady or
		not module.SetReady
	then
		return false
	end
	
	if type(module:GetType()) ~= "string" or
		type(module:GetName()) ~= "string" or
		type(module:GetDescription()) ~= "string" or
		type(module:IsReady()) ~= "boolean"
	then
		return false
	end
	
	return true
end

function NS:IterateModuleTypes()
	local iterator, t, key = pairs(NS._modules)
	
	return function()
		key = iterator(t, key)
		
		return key
	end
end

function NS:IterateModulesByType(type)
	if self._modules[type] then
		return pairs(self._modules[type])
	else
		return pairs(EMPTY_TABLE)
	end
end

-- module 

-- module functions
local function GetType(self) 
	return self._type
end

local function GetName(self) 
	return self._name
end

local function GetDescription(self) 
	return self._description
end

local function IsReady(self) 
	return self._ready
end

local function SetReady(self, ready) 
	self._ready = ready and true or false
end

-- factory method for module creation
function NS:CreateModule(type, name, description)
	local module = {
		-- members
		_type        = type,
		_name        = name,
		_description = description,
		_ready       = false,
		
		-- functions
		GetType        = GetType,
		GetName        = GetName,
		GetDescription = GetDescription,
		IsReady        = IsReady,
		SetReady       = SetReady,
	}
	
	return module
end
