local _G = _G

-- addon name and namespace
local ADDON, NS = ...

local Addon = LibStub("AceAddon-3.0"):GetAddon(ADDON)

-- get translations
local L = LibStub:GetLibrary("AceLocale-3.0"):GetLocale(ADDON)

-- lib hijack minimap
local LHM = LibStub:GetLibrary("LibHijackMinimap-1.0")

-- the module name
local MODULE      = "Minimap"
local DESCRIPTION = L["Module that provides direction arrow on the Minimap."]

-- local functions
local cos     = math.cos
local sin     = math.sin

local Minimap = Minimap

local GetCVar         = _G.GetCVar
local GetPlayerFacing = _G.GetPlayerFacing

local DIST_FROM_CENTER = 40

local ARROW = "Interface\\Addons\\"..ADDON.."\\minimap-magnet.tga"

local rotateMinimap = GetCVar("rotateMinimap") == "1"
local updateFrame = CreateFrame("Frame")

-- aux variables
local _

-- the module
local Outlet = NS:CreateOutlet(MODULE, DESCRIPTION)

-- the arrow frame
do
	Outlet.arrow = CreateFrame("Frame", ADDON .. "_" .. MODULE .. "_Minimap_Arrow", Minimap)
	Outlet.arrow:SetHeight(24)
	Outlet.arrow:SetWidth(24)
	Outlet.arrow:SetParent(Minimap)
	Outlet.arrow:SetFrameStrata(Minimap:GetFrameStrata())
	Outlet.arrow:SetFrameLevel(Minimap:GetFrameLevel() + 7)
	
	local texture = Outlet.arrow:CreateTexture(ADDON .. "_" .. MODULE .. "_Minimap_Arrow_Tex", "OVERLAY")
	Outlet.arrow.texture = texture
	texture:SetAllPoints(Outlet.arrow)
	texture:SetTexture(ARROW)
end

updateFrame:Hide()
updateFrame:SetScript("OnUpdate", function(frame, elapsed) Outlet:Refresh()	end)

-- LHM support
function Outlet:ReparentMinimap(frame)
	if not frame or frame == Minimap then
		return
	end

	Minimap = frame
	
	self.arrow:SetParent(Minimap)
	self.arrow:SetFrameStrata(Minimap:GetFrameStrata())
	self.arrow:SetFrameLevel(Minimap:GetFrameLevel() + 7)

	-- in case hijacker did not provide optional argument "ROTATE_MINIMAP" in SetCVar("rotateMinimap", value, "ROTATE_MINIMAP") so CVAR_UPDATE did not fire
	self:SetRotateMinimap(GetCVar("rotateMinimap") == "1")
	
	self:Refresh()
end

LHM:RegisterClient(ADDON .. "." .. MODULE, Outlet, "ReparentMinimap")

Outlet:SetReady(true)

-- register
NS:RegisterModule(Outlet)

-- handle rotate minimap
function Outlet:HandleActiveChanged()
	if self:IsActive() then
		self:SetRotateMinimap(GetCVar("rotateMinimap") == "1")
		
		updateFrame:RegisterEvent("CVAR_UPDATE")	
	else
		updateFrame:UnregisterEvent("CVAR_UPDATE")
	end
end

function Outlet:HandleCVarChanged(event, cvar, value)
	if cvar == "ROTATE_MINIMAP" then
		self:SetRotateMinimap(value == "1")
	end
end

updateFrame:SetScript("OnEvent", function(this, event, ...) Outlet:HandleCVarChanged(event, ...) end)

function Outlet:SetRotateMinimap(rotate)
	if rotateMinimap ~= rotate then
		rotateMinimap = rotate
		
		self:Refresh()
		
		if rotateMinimap then
			updateFrame:Show()
		else
			updateFrame:Hide()
		end
	end		
end

function Outlet:GetFacing()
	local facing = 0
	
	if rotateMinimap then
		if GetPlayerFacing then
			facing = -GetPlayerFacing()
		else
			facing = MiniMapCompassRing:GetFacing()
		end
	end
	
	return facing
end

-- implement functions for output
function Outlet:Refresh()
	if not self:IsOperational() then
		return
	end
	
	local magnitude, direction, sinDir, cosDir = Addon:GetForce()

	if magnitude == 0 then
		self:Clear()
		
		return
	end
	
	-- adjust facing
	if rotateMinimap then
		direction = direction + self:GetFacing()
		
		cosDir = cos(direction)
		sinDir = sin(direction)
	end
	
	-- determine rotated and scaled texture coordinates
	local gX = (-sinDir + cosDir) * 0.5
	local gY = (-sinDir - cosDir) * 0.5
	
	self.arrow.texture:SetTexCoord(
		0.5 - gX, 0.5 + gY,
		0.5 + gY, 0.5 + gX,
		0.5 - gY, 0.5 - gX,
		0.5 + gX, 0.5 - gY)
		
	self.arrow:ClearAllPoints()
	self.arrow:SetPoint("CENTER", Minimap, "CENTER", -DIST_FROM_CENTER * sinDir, DIST_FROM_CENTER * cosDir)
	
	self.arrow:Show()
end

function Outlet:Clear()
	if not self:IsReady() then
		return
	end

	if self.arrow:IsShown() then
		self.arrow:Hide()
	end
end

-- test
function Outlet:Debug(msg)
	Addon:Debug("(Outlet:" .. MODULE .. ") " .. msg)
end
