local _G = _G

-- addon name and namespace
local ADDON, NS = ...

local Addon = LibStub("AceAddon-3.0"):GetAddon(ADDON)

-- the Tooltip module
local Tooltip = Addon:NewModule("Tooltip")

-- get translations
local L = LibStub:GetLibrary("AceLocale-3.0"):GetLocale(ADDON)

-- LibNodeTypeInfo
local LibNTI = LibStub:GetLibrary("LibNodeTypeInfo-1.0")

-- tooltip library
local QT = LibStub:GetLibrary("LibQTip-1.0")

-- local functions
local ipairs   = ipairs 
local pairs    = pairs 
local strlower = strlower
local strfind  = strfind
local tinsert  = table.insert
local tsort    = table.sort

local GetExpansionLevel = _G.GetExpansionLevel

-- aux variables
local _

-- icons
local ICON_DISABLED = [[|TInterface\BUTTONS\UI-PlusButton-Disabled:16:16|t]]
local ICON_PLUS	    = [[|TInterface\BUTTONS\UI-PlusButton-Up:16:16|t]]
local ICON_MINUS    = [[|TInterface\BUTTONS\UI-MinusButton-Up:16:16|t]]

-- local variables
local tooltip

-- modules
local Options = nil

local function ToggleActive(cell, arg, button)
	Addon:ToggleActive()
	
	Tooltip:Refresh()
end

local function ToggleTracking(cell, arg, button)
	Addon:ToggleTracking(arg)
	
	Tooltip:Refresh()
end

local function ToggleDetails(cell, option)
	Options:ToggleTooltipDetails(option)
	
	Tooltip:Refresh()
end

local function ToggleBlacklistNode(cell, nodeId, button)
	Options:ToggleNodeTypeBlacklisted(nodeId)
	
	Tooltip:Refresh()
end

-- module handling
function Tooltip:OnInitialize()
	-- set module references
	Options = Addon:GetModule("Options")
end

function Tooltip:OnEnable()
	-- empty
end

function Tooltip:OnDisable()
	self:Remove()
end

function Tooltip:Create(obj)
	if not self:IsEnabled() then
		return
	end

	tooltip = QT:Acquire(ADDON.."TT", 4)
	
	tooltip:Hide()
	tooltip:Clear()
	tooltip:SetScale(1)
		
	self:Draw()

	tooltip:SetAutoHideDelay(.1, obj)
	tooltip:EnableMouse()
	tooltip:SmartAnchorTo(obj)
	tooltip:Show()
end

function Tooltip:Remove()
	if tooltip then
		tooltip:Hide()
		QT:Release(tooltip)
		tooltip = nil
	end
end

function Tooltip:Refresh()
	if not self:IsEnabled() then
		self:Remove()
		return
	end
	
	if not tooltip or not tooltip:IsShown() then
		return
	end
	
	self:Draw()
	
	if tooltip then
		tooltip:Show()
	end
end

function Tooltip:Draw()
	if not tooltip then
		return
	end

	tooltip:Hide()
	tooltip:Clear()

	-- module header
	local lineNum = tooltip:AddHeader(" ")
	tooltip:SetCell(lineNum, 1, NS:Colorize("White", Addon.FULLNAME ), "CENTER", tooltip:GetColumnCount())

	-- activity
	tooltip:AddLine(" ")
	lineNum = tooltip:AddLine(" ")
	tooltip:SetCell(lineNum, 1, L["Operation"], "LEFT")
	tooltip:SetCell(lineNum, 3, Addon:IsActive() and NS:Colorize("Green", L["enabled"]) or NS:Colorize("Red", L["disabled"]), "LEFT")
	
	tooltip:SetCellScript(lineNum, 3, "OnMouseDown", ToggleActive)
	
	-- inlet
	local label = L["None"]
	local operational = false
	local color = "GrayOut"
	
	local Inlets = Addon:GetModule("Inlets")
	
	local inlet = Inlets:GetActiveInlet()
	
	if inlet then
		label = inlet:GetName()
		operational = inlet:IsReady()
	end
	
	if operational then
		color = "White"
	else
		label = label .. " (" .. L["not operational"] .. ")"
	end
	
	lineNum = tooltip:AddLine(" ")
	tooltip:SetCell(lineNum, 1, L["Data Source"], "LEFT")
	tooltip:SetCell(lineNum, 3, NS:Colorize(color, label), "LEFT")
		
	-- state
	local state = Addon:GetState()
	local label = ""
	
	if state == Addon.INACTIVE then
		label = NS:Colorize("GrayOut", L["Inactive"])
	elseif state == Addon.IDLE then
		label = NS:Colorize("White",   L["Idle"])
	elseif state == Addon.PAUSED then
		label = NS:Colorize("Yellow",  L["Paused"])
	elseif state == Addon.TRACKING then
		label = NS:Colorize("Green",   L["Tracking"])
	end
	
	lineNum = tooltip:AddLine(" ")
	tooltip:SetCell(lineNum, 1, L["State"], "LEFT")
	tooltip:SetCell(lineNum, 3, label, "LEFT")
	
	local visitedInfo = NS:Colorize("Gray", "--")
	
	local total, unvisited = 0, 0
	
	_, _, _, _, _, total, visited = Addon:GetForce()
	
	if total > 0 then
		local percentage = visited*100/total
		visitedInfo = NS:ColorizeByValue(percentage, 0, 100, string.format(L["%.2f%% (%d of %d)"], percentage, visited, total))
	end
	
	lineNum = tooltip:AddLine(" ")
	tooltip:SetCell(lineNum, 1, NS:Colorize(name, L["Visited"]), "LEFT")				
	tooltip:SetCell(lineNum, 3, visitedInfo, "LEFT")
		
	-- tracking
	local count = 0
	
	for profession, tracking in Addon:IterateTrackingSkills() do
		if tracking.ready then
			count = count + 1
			
			tooltip:AddLine(" ")

			local name = Addon.PROFESSIONS[profession].name

			if tracking.active then
				label = NS:Colorize("Green", L["enabled"])
			else
				label = NS:Colorize("Red", L["disabled"])
			end
						
			lineNum = tooltip:AddLine(" ")
			tooltip:SetCell(lineNum, 1, NS:Colorize(name, name), "LEFT")
			tooltip:SetCell(lineNum, 3, label, "LEFT")
		
			-- activate/deactivate tracking
			tooltip:SetCellScript(lineNum, 3, "OnMouseDown", ToggleTracking, profession)
			
			-- details (+/-) button
			local details = not Options:GetTooltipDetails("hide" .. name .. "Details")
			
			tooltip:SetCell(lineNum, 4, details and ICON_MINUS or ICON_PLUS, "RIGHT")
			tooltip:SetCellScript(lineNum, 4, "OnMouseDown", ToggleDetails, "hide" .. name .. "Details")
			
			if details then
				local skill = Addon:GetSkill(profession)
				local zoneName, zoneID = Addon:GetZoneInfo()
			
				lineNum = tooltip:AddLine(" ")
				tooltip:SetCell(lineNum, 1, NS:Colorize(name, L["Skill"]), "LEFT")				
				tooltip:SetCell(lineNum, 3, skill, "LEFT")
				
				-- NOTE: use nodes from current zone only or all that are currently found while tracking?
				tooltip:AddLine(" ")
				lineNum = tooltip:AddLine(" ")
				tooltip:SetCell(lineNum, 1, NS:Colorize(name, L["Nodes in current zone:"]), "LEFT", 3)

				local nodes = LibNTI:GetNodesForZone(profession, zoneID)
							
				if #nodes == 0 then
					lineNum = tooltip:AddLine(" ")
					tooltip:SetCell(lineNum, 1, L["None"], "LEFT")
				else			
					table.sort(nodes, NS.compareNodes)
					
					local expansion = GetExpansionLevel()
					
					for index, nodeId in ipairs(nodes) do												
						local name     = LibNTI:GetName(profession, nodeId)
						local color    = LibNTI:GetExpansion(profession, nodeId) > expansion and "Red" or "White"
						local disabled = Options:IsNodeTypeBlacklisted(nodeId)
						
						lineNum = tooltip:AddLine(" ")

						tooltip:SetCell(lineNum, 1, NS:Colorize(color, name, "LEFT"))
						tooltip:SetCell(lineNum, 2, " ", "RIGHT")
						tooltip:SetCell(lineNum, 3, disabled and NS:Colorize("Red", L["disabled"]) or NS:Colorize("Green", L["enabled"]), "LEFT")
						
						tooltip:SetCellScript(lineNum, 3, "OnMouseDown", ToggleBlacklistNode, nodeId)
					end
				end
			end -- details
		end -- ready
	end
	
	if count == 0 then
		tooltip:AddLine(" ")

		lineNum = tooltip:AddLine(" ")
		tooltip:SetCell(lineNum, 1, NS:Colorize("GrayOut", L["You don't have any gathering profession."]), "LEFT", 3)
	end
	
	-- hint to show options
	if not Addon:GetSetting("hideHint") then
		tooltip:AddLine(" ")
		lineNum = tooltip:AddLine(" ")
		tooltip:SetCell(lineNum, 1, NS:Colorize("Brownish", L["Left-Click"] .. ":") .. " " .. NS:Colorize("Yellow", L["Activate/deactivate operation."] ), "LEFT", tooltip:GetColumnCount())
		lineNum = tooltip:AddLine(" ")
		tooltip:SetCell(lineNum, 1, NS:Colorize("Brownish", L["Right-Click"] .. ":") .. " " .. NS:Colorize("Yellow", L["Open option menu."] ), "LEFT", tooltip:GetColumnCount())
		lineNum = tooltip:AddLine(" ")
		tooltip:SetCell(lineNum, 1, NS:Colorize("Brownish", L["Left-Click Tip"] .. ":") .. " " .. NS:Colorize("Yellow", L["Toggle selected setting."] ), "LEFT", tooltip:GetColumnCount())
	end
end

-- test
function Tooltip:Debug(msg)
	Addon:Debug("(Tooltip) " .. msg)
end
